<?php

namespace Drupal\entity_print_typeset\Plugin\EntityPrint\PrintEngine;

use Drupal\entity_print\Plugin\EntityPrint\PrintEngine\PdfEngineBase;
use Drupal\entity_print\Plugin\ExportTypeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Typesetsh\HtmlToPdf;
use Typesetsh\Result;

/**
 * PHP typeset.sh plugin.
 *
 * @PrintEngine(
 *   id = "typeset",
 *   label = @Translation("Typeset.sh"),
 *   export_type = "pdf"
 * )
 *
 * To use this implementation you will need the Php Wkhtmltopdf library, simply
 * run:
 *
 * @code
 *    composer config repositories.typesetsh composer https://packages.typeset.sh
 *    composer require "typesetsh/typeset.sh"
 *
 * @endcode
 */
class Typeset extends PdfEngineBase {

  /**
   * The typeset.sh object.
   *
   * @var \Typesetsh\HtmlToPdf
   */
  protected HtmlToPdf $htmlToPdf;

  /**
   * The typeset.sh result.
   *
   * @var \Typesetsh\Result|null
   */
  protected ?Result $result = NULL;

  /**
   * Ctor.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ExportTypeInterface $export_type) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $export_type);
    $this->htmlToPdf = new HtmlToPdf();
  }

  /**
   * Create instance.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    // @todo Change the autogenerated stub
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  protected function getPaperSizes() {
    return [
      'a4' => 'A4',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function addPage($content) {
    $this->result = $this->htmlToPdf->render($content);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function send($filename, $force_download = TRUE) {
    $disposition = $force_download ? 'attachment' : 'inline';
    header('Content-Type: application/pdf');
    header('Content-Disposition: ' . $disposition . '; filename="' . $filename . '"');

    print $this->result->asString();
  }

  /**
   * {@inheritdoc}
   */
  public function getBlob() {
    return $this->result->asString();
  }

  /**
   * {@inheritdoc}
   */
  public static function dependenciesAvailable() {
    $available = class_exists('Typesetsh\HtmlToPdf');
    return $available;
  }

  /**
   * {@inheritdoc}
   */
  public function getPrintObject() {
    return $this->htmlToPdf;
  }

}
